@@ -76,10 +76,11 @@ geom_label_repel <- function(
 }
 
 #' GeomLabelRepel
-#' @rdname ggrepel-ggproto
+#' @rdname ggrepel
 #' @format NULL
 #' @usage NULL
 #' @seealso \link[ggplot2]{GeomLabel} from the ggplot2 package.
+#' @keywords internal
 #' @export
 GeomLabelRepel <- ggproto(
   "GeomLabelRepel", Geom,
@@ -127,13 +128,19 @@ GeomLabelRepel <- ggproto(
       return()
     }
 
-    # position_nudge_repel() should have added these columns.
+    # if needed rename columns using our convention
     for (this_dim in c("x", "y")) {
+      this_orig <- sprintf("%s_orig", this_dim)
       this_nudge <- sprintf("nudge_%s", this_dim)
       if (!this_nudge %in% colnames(data)) {
         data[[this_nudge]] <- data[[this_dim]]
+        if (this_orig %in% colnames(data)) {
+          data[[this_dim]] <- data[[this_orig]]
+          data[[this_orig]] <- NULL
+        }
       }
     }
+
     # Transform the nudges to the panel scales.
     nudges <- data.frame(x = data$nudge_x, y = data$nudge_y)
     nudges <- coord$transform(nudges, panel_scales)
@@ -251,8 +258,8 @@ makeContent.labelrepeltree <- function(x) {
   })
 
   # Make the repulsion reproducible if desired.
-  if (is.null(x$seed) || !is.na(x$seed)) {
-      set.seed(x$seed)
+  if (!is.null(x$seed) && is.na(x$seed)) {
+    x$seed <- sample.int(.Machine$integer.max, 1L)
   }
 
   # The points are represented by circles.
@@ -274,7 +281,7 @@ makeContent.labelrepeltree <- function(x) {
   ) / 13
 
   # Repel overlapping bounding boxes away from each other.
-  repel <- repel_boxes2(
+  repel <- with_seed_null(x$seed, repel_boxes2(
     data_points     = as.matrix(x$data[,c("x","y")]),
     point_size      = point_size,
     point_padding_x = point_padding,
@@ -291,7 +298,7 @@ makeContent.labelrepeltree <- function(x) {
     max_overlaps    = x$max.overlaps,
     direction       = x$direction,
     verbose         = x$verbose
-  )
+  ))
 
   if (any(repel$too_many_overlaps)) {
     warn(
